@@ -1,15 +1,5 @@
 ## this file contains some basic methods definitions
 
-## functions to extract or call the components of a "pomp" object
-setGeneric("data.array",function(object,...)standardGeneric("data.array"))
-setGeneric("obs",function(object,...)standardGeneric("obs"))
-setGeneric("time<-",function(object,...,value)standardGeneric("time<-"))  
-setGeneric("coef<-",function(object,...,value)standardGeneric("coef<-"))
-setGeneric("states",function(object,...)standardGeneric("states"))
-setGeneric("timezero",function(object,...)standardGeneric("timezero"))
-setGeneric("timezero<-",function(object,...,value)standardGeneric("timezero<-"))
-setGeneric("partrans",function(object,params,dir=c("forward","inverse"),...)standardGeneric("partrans"))
-
 ## 'coerce' method: allows for coercion of a "pomp" object to a data-frame
 setAs(
       from="pomp",
@@ -47,33 +37,19 @@ setMethod("partrans","pomp",
           partrans.internal(object=object,params=params,dir=dir,...)
           )
 
-## a simple method to extract the data array
-setMethod(
-          "data.array",
-          "pomp",
-          function (object, vars, ...) {
-            varnames <- rownames(object@data)
-            if (missing(vars))
-              vars <- varnames
-            else if (!all(vars%in%varnames))
-              stop("some elements of ",sQuote("vars")," correspond to no observed variable")
-            object@data[vars,,drop=FALSE]
-          }
-          )
+
+obs.internal <- function (object, vars, ...) {
+  varnames <- rownames(object@data)
+  if (missing(vars))
+    vars <- varnames
+  else if (!all(vars%in%varnames))
+    stop("some elements of ",sQuote("vars")," correspond to no observed variable")
+  object@data[vars,,drop=FALSE]
+}
 
 ## a simple method to extract the data array
-setMethod(
-          "obs",
-          "pomp",
-          function (object, vars, ...) {
-            varnames <- rownames(object@data)
-            if (missing(vars))
-              vars <- varnames
-            else if (!all(vars%in%varnames))
-              stop("some elements of ",sQuote("vars")," correspond to no observed variable")
-            object@data[vars,,drop=FALSE]
-          }
-          )
+setMethod("obs","pomp",obs.internal)
+setMethod("data.array","pomp",obs.internal)
 
 ## a simple method to extract the array of states
 setMethod(
@@ -293,6 +269,10 @@ setMethod(
             show(object@rmeasure)
             cat("measurement model density, dmeasure = \n")
             show(object@dmeasure)
+            cat("prior simulator, rprior = \n")
+            show(object@rprior)
+            cat("prior density, dprior = \n")
+            show(object@dprior)
             if (!is.na(object@skeleton.type)) {
               cat("skeleton (",object@skeleton.type,") = \n")
               show(object@skeleton)
