@@ -1,15 +1,18 @@
 require(pomp)
 
 ## negative binomial measurement model
-dmeas <- "
-  double prob = theta/(theta+rho*incid);
-  lik = dnbinom(cases,theta,prob,give_log);
-"
-rmeas <- "
-  double prob = theta/(theta+rho*incid);
-  cases = rnbinom(theta,prob);
-"
+## E[cases|incid] = rho*incid
+## Var[cases|incid] = rho*incid*(1+rho*incid/theta)
+rmeas <- '
+  cases = rnbinom_mu(theta,rho*incid);
+'
+
+dmeas <- '
+  lik = dnbinom_mu(cases,theta,rho*incid,give_log);
+'
+
 ## SIR process model with extra-demographic stochasticity
+## and seasonal transmission
 step.fn <- '
   int nrate = 6;
   double rate[nrate];		// transition rates
@@ -18,6 +21,7 @@ step.fn <- '
   double dW;			// white noise increment
   int k;
 
+  // seasonality in transmission
   beta = beta1*seas1+beta2*seas2+beta3*seas3;
 
   // compute the environmental stochasticity
@@ -41,9 +45,10 @@ step.fn <- '
   S += trans[0]-trans[1]-trans[2];
   I += trans[1]-trans[3]-trans[4];
   R += trans[3]-trans[5];
-  incid += trans[3];		// cases are cumulative recoveries
+  incid += trans[3];	// incidence is cumulative recoveries
   if (beta_sd > 0.0) W += (dW-dt)/beta_sd; // increment has mean = 0, variance = dt
 '
+
 skel <- '
   int nrate = 6;
   double rate[nrate];		// transition rates
@@ -77,9 +82,12 @@ skel <- '
   Dincid = term[3];		// accumulate the new I->R transitions
   DW = 0;
 '
+
 ## parameter transformations
+## note we use barycentric coordinates for the initial conditions
+## the success of this depends on S0, I0, R0 being in
+## adjacent memory locations, in that order
 partrans <- "
-  double sum;
   Tgamma = exp(gamma);
   Tmu = exp(mu);
   Tiota = exp(iota);
@@ -89,16 +97,10 @@ partrans <- "
   Tbeta_sd = exp(beta_sd);
   Trho = expit(rho);
   Ttheta = exp(theta);
-  TS_0 = exp(S_0);
-  TI_0 = exp(I_0);
-  TR_0 = exp(R_0);
-  sum = TS_0+TI_0+TR_0;
-  TS_0 /= sum;
-  TI_0 /= sum;
-  TR_0 /= sum;
+  from_log_barycentric(&TS_0,&S_0,3);
 "
+
 paruntrans <- "
-  double sum;
   Tgamma = log(gamma);
   Tmu = log(mu);
   Tiota = log(iota);
@@ -108,10 +110,7 @@ paruntrans <- "
   Tbeta_sd = log(beta_sd);
   Trho = logit(rho);
   Ttheta = log(theta);
-  sum = S_0+I_0+R_0;
-  TS_0 = log(S_0/sum);
-  TI_0 = log(I_0/sum);
-  TR_0 = log(R_0/sum);
+  to_log_barycentric(&TS_0,&S_0,3);
 "
 
 data(LondonYorke)
@@ -170,7 +169,7 @@ coef(po) <- c(
               beta1=120,beta2=140,beta3=100,
               beta.sd=0.01,
               popsize=5e6,
-              rho=0.2,theta=0.001,
+              rho=0.1,theta=1,
               S.0=0.22,I.0=0.0018,R.0=0.78
               )
 
@@ -189,20 +188,3 @@ toc <- Sys.time()
 print(toc-tic)
 
 plot(incid~time,data=x,col=as.factor(x$sim),pch=16)
-
-coef(po) <- coef(
-                 traj.match(
-                            pomp(
-                                 window(po,end=1930)
-                                 ## window(po,end=1930),
-                                 ## measurement.model=cases~norm(mean=rho*incid,sd=100)
-                                 ),
-                            est=c("S.0","I.0","R.0"),
-                            transform=TRUE
-                            )
-                 )
-
-pf <- pfilter(po,Np=1000,max.fail=100)
-print(round(logLik(pf),1))
-
-dyn.unload("SIR.so")
