@@ -5,21 +5,21 @@
 
 #include <R.h>
 #include <Rmath.h>
-#include <Rdefines.h>
+#include <Rinternals.h>
 #include <R_ext/Rdynload.h>
 
-// facility for extracting R objects from the 'userdata' slot
+// FACILITIES FOR EXTRACTING R OBJECTS FROM THE 'USERDATA' SLOT
 const SEXP get_pomp_userdata (const char *name);
 const int *get_pomp_userdata_int (const char *name);
 const double *get_pomp_userdata_double (const char *name);
 
-// facility for computing evaluating a basis of periodic bsplines
+// FACILITY FOR EVALUATING A SET OF PERIODIC BSPLINE BASIS FUNCTIONS
 void periodic_bspline_basis_eval (double x, double period, int degree, int nbasis, double *y);
 
-// Prototype for parameter transformation function.
+// PROTOTYPE FOR PARAMETER TRANSFORMATION FUNCTION.
 typedef void pomp_transform_fn (double *pt, double *p, int *parindex);
 
-// Prototype for stochastic simulation algorithm reaction-rate function, as used by "gillespie.sim":
+// PROTOTYPE FOR STOCHASTIC SIMULATION ALGORITHM REACTION-RATE FUNCTION, AS USED BY "GILLESPIE.SIM":
 typedef double pomp_ssa_rate_fn(int j, double t, const double *x, const double *p,
 				int *stateindex, int *parindex, int *covindex,
 				int ncovar, double *covars);
@@ -40,7 +40,7 @@ typedef double pomp_ssa_rate_fn(int j, double t, const double *x, const double *
 //                from the covariate table supplied to 'SSA.simulator'
 //  returns the rate of the j-th reaction
 
-// Prototype for one-step simulator, as used by "euler.sim" and "onestep.sim":
+// PROTOTYPE FOR ONE-STEP SIMULATOR, AS USED BY "EULER.SIM" AND "ONESTEP.SIM":
 typedef void pomp_onestep_sim(double *x, const double *p, 
 			      const int *stateindex, const int *parindex, const int *covindex,
 			      int ncovars, const double *covars,
@@ -68,7 +68,7 @@ typedef void pomp_onestep_sim(double *x, const double *p,
 //     Inclusion of these calls in the user-defined function may result in significant slowdown.
 
 
-// Prototype for one-step log probability density function, as used by "onestep.dens":
+// PROTOTYPE FOR ONE-STEP LOG PROBABILITY DENSITY FUNCTION, AS USED BY "ONESTEP.DENS":
 typedef void pomp_onestep_pdf(double *f, 
 			      double *x1, double *x2, double t1, double t2, const double *p, 
 			      const int *stateindex, const int *parindex, const int *covindex,
@@ -92,7 +92,7 @@ typedef void pomp_onestep_pdf(double *f,
 //  on output:
 // f          = pointer to the probability density (a single scalar)
 
-// Prototype for deterministic skeleton evaluation
+// PROTOTYPE FOR DETERMINISTIC SKELETON EVALUATION
 typedef void pomp_skeleton (double *f, double *x, double *p, 
 			    int *stateindex, int *parindex, int *covindex, 
 			    int ncovars, double *covars, double t);
@@ -114,7 +114,7 @@ typedef void pomp_skeleton (double *f, double *x, double *p,
 //  on output:
 // f          = pointer to value of the map or vectorfield (a vector of the same length as 'x')
 
-// Prototype for measurement model simulation
+// PROTOTYPE FOR MEASUREMENT MODEL SIMULATION
 typedef void pomp_measure_model_simulator (double *y, double *x, double *p, 
 					   int *obsindex, int *stateindex, int *parindex, int *covindex,
 					   int ncovars, double *covars, double t);
@@ -142,7 +142,7 @@ typedef void pomp_measure_model_simulator (double *y, double *x, double *p,
 //     Inclusion of these calls in the user-defined function may result in significant slowdown.
 
 
-// Prototype for measurement model density evaluator
+// PROTOTYPE FOR MEASUREMENT MODEL DENSITY EVALUATOR
 typedef void pomp_measure_model_density (double *lik, double *y, double *x, double *p, int give_log,
 					 int *obsindex, int *stateindex, int *parindex, int *covindex,
 					 int ncovars, double *covars, double t);
@@ -167,6 +167,32 @@ typedef void pomp_measure_model_density (double *lik, double *y, double *x, doub
 //  on output:
 // lik        = pointer to scalar containing (log) likelihood
 
+// PROTOTYPE FOR PRIOR SIMULATION
+typedef void pomp_rprior (double *p, int *parindex);
+// Description:
+//  on input:
+// p          = pointer to parameter vector
+// parindex   = pointer to vector of integers indexing the parameters in 'p' in the order specified by 
+//                the 'paramnames' slot
+//  on output:
+// p          = pointer to vector containing draws from the prior
+//
+// NB: There is no need to call GetRNGstate() or PutRNGstate() in the body of the user-defined function.
+//     The RNG is initialized before any call to this function, and the RNG state is written afterward.
+//     Inclusion of these calls in the user-defined function may result in significant slowdown.
+
+// PROTOTYPE FOR PRIOR DENSITY EVALUATION
+typedef void pomp_dprior (double *lik, double *p, int give_log, int *parindex);
+// Description:
+//  on input:
+// p          = pointer to parameter vector
+// give_log   = should the log likelihood be returned?
+// parindex   = pointer to vector of integers indexing the parameters in 'p' in the order specified by 
+//                the 'paramnames' slot
+//  on output:
+// lik        = pointer to vector containing likelihoods
+
+// UTILITY FOR EXPONENTIAL/GEOMETRIC RATE CONVERSION
 // This function computes r such that if
 // N ~ geometric(prob=1-exp(-r dt)) and T ~ exponential(rate=R),
 // then E[N dt] = E[T]
@@ -177,6 +203,7 @@ static R_INLINE double exp2geom_rate_correction (double R, double dt) {
   return (dt > 0) ? log(1.0+R*dt)/dt : R;
 }
 
+// UTILITY FOR GAMMA WHITENOISE
 // This function draws a random increment of a gamma whitenoise process.
 // This will have expectation=dt and variance=(sigma^2*dt)
 // If dW = rgammawn(sigma,dt), then 
@@ -189,10 +216,19 @@ static R_INLINE double rgammawn (double sigma, double dt) {
   return (sigmasq > 0) ? rgamma(dt/sigmasq,sigmasq) : dt;
 }
 
+// VECTOR DOT-PRODUCT UTILITY
 // facility for computing the inner product of 
 // a vector of parameters ('coef') against a vector of basis-function values ('basis')
-double dot_product (int dim, const double *basis, const double *coef);
+static R_INLINE double dot_product (int dim, const double *basis, const double *coef) {
+  int j;
+  double trans = 0.0;
+  for (j = 0; j < dim; j++)
+    trans += coef[j]*basis[j];
+  return(trans);
+}
 
+
+// LOGIT AND INVERSE LOGIT TRANSFORMATIONS
 static R_INLINE double logit (double p) {
   return log(p/(1.0-p));
 }
@@ -201,23 +237,127 @@ static R_INLINE double expit (double x) {
   return 1.0/(1.0+exp(-x));
 }
 
-// prototypes for C-level access to Euler-multinomial distribution functions
+// C-LEVEL DEFINITIONS OF EULER-MULTINOMIAL DISTRIBUTION FUNCTIONS
 
 // simulate Euler-multinomial transitions
 // NB: 'reulermultinom' does not call GetRNGstate() and PutRNGstate() internally
 // this must be done by the calling program
 // But note that when reulermultinom is called inside a pomp 'rprocess', there is no need to call
 // {Get,Put}RNGState() as this is handled by pomp
-void reulermultinom (int m, double size, double *rate, double dt, double *trans);
+static void reulermultinom (int m, double size, double *rate, 
+			    double dt, double *trans) {
+  double p = 0.0;
+  int j, k;
+  if ((size < 0.0) || (dt < 0.0) || (floor(size+0.5) != size)) {
+    for (k = 0; k < m; k++) trans[k] = R_NaN;
+    return;
+  }  
+  for (k = 0; k < m; k++) {
+    if (rate[k] < 0.0) {
+      for (j = 0; j < m; j++) trans[j] = R_NaN;
+      return;
+    }
+    p += rate[k]; // total event rate
+  }
+  if (p > 0.0) {
+    size = rbinom(size,1-exp(-p*dt)); // total number of events
+    if (!(R_FINITE(size)))
+      warning("reulermultinom: result of binomial draw is not finite");
+    m -= 1;
+    for (k = 0; k < m; k++) {
+      if (rate[k] > p) p = rate[k];
+      trans[k] = ((size > 0) && (p > 0)) ? rbinom(size,rate[k]/p) : 0;
+      if (!(R_FINITE(size)&&R_FINITE(p)&&R_FINITE(rate[k])&&R_FINITE(trans[k])))
+	warning("reulermultinom: result of binomial draw is not finite");
+      size -= trans[k];
+      p -= rate[k];
+    }
+    trans[m] = size;
+  } else {
+    for (k = 0; k < m; k++) trans[k] = 0.0;
+  }
+}
 
-// compute probabilities of eulermultinomial transitions
-double deulermultinom (int m, double size, double *rate, double dt, double *trans, int give_log);
+// COMPUTE PROBABILITIES OF EULER-MULTINOMIAL TRANSITIONS
+static double deulermultinom (int m, double size, double *rate, double dt, 
+			      double *trans, int give_log) {
+  double p = 0.0;
+  double n = 0.0;
+  double ff = 0.0;
+  int k;
+  if ((dt < 0.0) || (size < 0.0) || (floor(size+0.5) != size)) {
+    warning("NaNs produced");
+    return R_NaN;
+  }
+  for (k = 0; k < m; k++) {
+    if (rate[k] < 0.0) {
+      warning("NaNs produced");
+      return R_NaN;
+    }
+    if (trans[k] < 0.0) {
+      ff = (give_log) ? R_NegInf: 0.0;
+      return ff;
+    }
+    p += rate[k]; // total event rate
+    n += trans[k]; // total number of events
+  }
+  if (n > size) {
+    ff = (give_log) ? R_NegInf: 0.0;
+    return ff;
+  }
+  ff = dbinom(n,size,1-exp(-p*dt),1); // total number of events
+  m -= 1;
+  for (k = 0; k < m; k++) {
+    if ((n > 0) && (p > 0)) {
+      if (rate[k] > p) p = rate[k];
+      ff += dbinom(trans[k],n,rate[k]/p,1);
+    } else if (trans[k] > 0.0) {
+      ff = R_NegInf;
+      return ff;
+    }
+    n -= trans[k];
+    p -= rate[k];
+  }
+  ff = (give_log) ? ff : exp(ff);
+  return ff;
+}
+
+// C-LEVEL DEFINITIONS OF LOG-BARYCENTRIC TRANSFORMATION.
+// USEFUL FOR WORKING WITH PARAMETERS CONSTRAINED TO SUM TO 1
+
+// TRANSFORMS TO LOG BARYCENTRIC COORDINATES
+// on input:
+// x =  pointer to vector of parameters to be tranformed to 
+//      log barycentric coordinates,
+// n =  length of vector.
+// on output:
+// xt = pointer to vector of log barycentric coordinates
+static R_INLINE void to_log_barycentric (double *xt, const double *x, int n) {
+  double sum;
+  int i;
+  for (i = 0, sum = 0.0; i < n; i++) sum += x[i];
+  for (i = 0; i < n; i++) xt[i] = log(x[i]/sum);
+}
+
+// TRANSFORMS FROM LOG BARYCENTRIC COORDINATES
+// on input:
+// x =  pointer to vector of parameters in log barycentric coordinates,
+// n =  length of vector.
+// on output:
+// xt = pointer to vector of coordinates on unit simplex
+static R_INLINE void from_log_barycentric (double *xt, const double *x, int n) {
+  double sum;
+  int i;
+  for (i = 0, sum = 0.0; i < n; i++) sum += (xt[i] = exp(x[i]));
+  for (i = 0; i < n; i++) xt[i] /= sum;
+}
 
 static R_INLINE double rbetabinom (double size, double prob, double theta) {
   return rbinom(size,rbeta(prob*theta,(1.0-prob)*theta));
 }
 
-static R_INLINE double dbetabinom (double x, double size, double prob, double theta, int give_log) {
+static R_INLINE double dbetabinom (double x, double size, double prob, 
+double theta, int give_log) {
   double a = theta*prob;
   double b = theta*(1.0-prob);
   double f = lchoose(size,x)-lbeta(a,b)+lbeta(a+x,b+size-x);
@@ -229,7 +369,8 @@ static R_INLINE double rbetanbinom (double mu, double size, double theta) {
   return rnbinom(size,rbeta(prob*theta,(1.0-prob)*theta));
 }
 
-static R_INLINE double dbetanbinom (double x, double mu, double size, double theta, int give_log) {
+static R_INLINE double dbetanbinom (double x, double mu, double size, 
+double theta, int give_log) {
   double prob = size/(size+mu);
   double a = theta*prob;
   double b = theta*(1.0-prob);
